function final_variance=imvariance(initial_image)
//
// DESCRIPTION
// Computes the value of the variance for each point of an image,
// ie the sum of the absolute value of the differences between 
// the central pixel and its neighbours
// variance=sum(|x(neighbour)-x(central)|)
// 
// A low variance value means a pixel is not very different
// from its neigbours (in all the directions).
//
// This property can be used to unwrap phased images:
// in case of a "path-following algorithm", the variance
// can be a "merit function"
// used to determine which pixels should be connected first.
//
// This algorithm calculates the variance everywhere,
// even on the edges.
// In some cases, consider multiplying by a mask like this 
// [8/3 8/5 8/5...;
//  8/5   1   1...;
// so that edge values are really significatives.
//
// AUTHOR
//    Jocelyn DRUEL  <jocelyn.druel1@libertysurf.fr>
//    Labo Photonique, Lycee Gustave Eiffel
//    59280 ARMENTIERES (FRANCE)
//
// LICENSE: GNU GPL
// $Revision: 1.1 $ $Date: 2003/04/05 20:45:50 $


if argn(2)==0 then
  error("imvariance(image)")
end;

if typeof(initial_image) == 'hypermat'
   warning(' this function works only with gray level images. Use im2gray for example.')
end


final_variance=zeros(initial_image); 
col=size(final_variance,'c');
line=size(final_variance,'r');

// One easy way to calculate variance is to use "for" loops
// but it's not very efficient with Scilab.
// I prefered to make simple operations between the original image
// and a shifted image.
// This operation is repeated 8 times (left, up, down, etc... )
// to treat all the 8 neighbours
// 1st: we shift the matrix
// 2nd: we take care of the edges

// shift to the left
shifted_matrix=initial_image(:,2:$); //shift the matrix to the left
shifted_matrix(:,col)=initial_image(:,col);//take care of edges
final_variance=final_variance+abs(initial_image-shifted_matrix);

// shift up
shifted_matrix=initial_image(2:$,:);
shifted_matrix(line,:)=initial_image(line,:);
final_variance=final_variance+abs(initial_image-shifted_matrix);

// shift to the right
shifted_matrix(:,2:$)=initial_image(:,1:$-1);
shifted_matrix(:,1)=initial_image(:,1);
final_variance=final_variance+abs(initial_image-shifted_matrix);

// shift down
shifted_matrix(2:$,:)=initial_image(1:$-1,:);
shifted_matrix(1,:)=initial_image(1,:);
final_variance=final_variance+abs(initial_image-shifted_matrix);


// The diagonals
//
// shift to the left up
shifted_matrix(1:$-1,1:$-1)=initial_image(2:$,2:$);
shifted_matrix(:,col)=initial_image(:,col);
shifted_matrix(line,:)=initial_image(line,:);
final_variance=final_variance+abs(initial_image-shifted_matrix);

// shift to the left down
shifted_matrix(2:$,1:$-1)=initial_image(1:$-1,2:$);
shifted_matrix(:,col)=initial_image(:,col);
shifted_matrix(1,:)=initial_image(1,:);
final_variance=final_variance+abs(initial_image-shifted_matrix);

// shift to the right up
shifted_matrix(1:$-1,2:$)=initial_image(2:$,1:$-1);
shifted_matrix(:,1)=initial_image(:,1);
shifted_matrix(line,:)=initial_image(line,:);
final_variance=final_variance+abs(initial_image-shifted_matrix);

// shift to the right down
shifted_matrix(2:$,2:$)=initial_image(1:$-1,1:$-1);
shifted_matrix(:,1)=initial_image(:,1);
shifted_matrix(1,:)=initial_image(1,:);
final_variance=final_variance+abs(initial_image-shifted_matrix);

// All the 8 neighbours have been treated. End

endfunction;



//
// -------------------------------------------------------------------------
// SIP - Scilab Image Processing toolbox
// Copyright (C) 2002-2004  Ricardo Fabbri
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// -------------------------------------------------------------------------
//
