function [puw,jumps]=unwrapp(pw,mf,line_start,column_start)

// Description:
// Unwraps a phased image (gray levels) by path following:
// 1) the quality of each pixel is estimated: the result
// is stored in a "merit function" (mf).
// Some merit functions can be: laplacien, variance...
// A good quality pixel has a low merit function.
// 2) We begin from a point of good quality. It is called "integrator point".
// 3) we rely it to its best quality neighbour, which becomes the new
// "integrator point".
// 4) If the difference between phases of the 2 points is higher (or lower)
// than a threshold, then we consider there's a phase jump.
// 5) we continue until all points are treated.
//
// (pw means "phase wrapped" and puw means "phase unwrapped")
//
// NB: we store the path in a matrix ("path"). If we go in a baggin
// (all neighbours have been treated), then we restart from the beginning until
// we find a pixel with an unvisited neighbour.
//
// This algorithm is a bit slow because of the number of the
// pixels to be treated: we search the best pixel for each pixel.
// A 512*512 pixels result in more than 256 000 loops.
// A C coding could be faster... Volunteers are welcome...
// 
// AUTHOR
//    Jocelyn DRUEL  <jocelyn.druel1@libertysurf.fr>
//    Labo Photonique, Lycee Gustave Eiffel
//    59280 ARMENTIERES (FRANCE)
//
// REFERENCES
// David VENET:
// "techniques automatiques de raccordement de phase"
// Memoire de l'universite de Lausanne:
//
// LICENSE: GNU GPL
// $Revision: 1.4 $ $Date: 2004/02/07 01:04:14 $
//
// Changelog:
// Function adapted to the new SIp convention of representing all 
// images in 0-1 range.
//
// 1.3: points with a merit function (mf) >= vval (default: vval=3000)
// will not be demodulated (they are considered as too bad to be demodulated.
// The "shift" variable was changed to "shift1" to avoid conflicts
// with the homonym Scilab function
//

if (argn(2)==0|argn(2)>4) then   
   error("unwrapp(wrapped_phase,merit_function[,line_start,column_start])"); 
end;


if typeof(pw) == 'hypermat'
   warning(' this function works only with gray level images. ...
 Use im2gray for example.')
end


if ~exists('line_start','local') then
	//find the best starting point
	lcenter=round(size(pw,'r')/2);//center of image
	ccenter=round(size(pw,'c')/2);

   //we search the center in 1/10 of the image
	radius=round(min(size(pw,'r'),size(pw,'c'))/10);
	mmatrix=mf(lcenter-radius:lcenter+radius,ccenter-radius:ccenter+radius);
	[val_min,position]=min(mmatrix);

	line_start=lcenter+(position(1)-radius-1)
	column_start=ccenter+(position(2)-radius-1)
	//end of best starting point
end; //if exist




//memorizing path followed
path(1:size(pw,'r')*size(pw,'c'))=0;

//starting
l=line_start;
c=column_start;



//we don't treat borders
vval=3000;//visited value: mf=vval if pixel was visited
mf(1,:)=vval;mf($,:)=vval;mf(:,1)=vval;mf(:,$)=vval;
seuil_mf=9*vval-1;

//memorize phase jumps in a matrix
jumps=zeros(pw);

shift1=0;//phase shift (integer*256)
threshold=0.5;
mf(l,c)=vval;//the starting pixel is considered as visited
nb_treated_pts=1;

//find the points with mf>=vval: these points will be ignored by demodulation
visit1=find(mf>(vval-1));
[visitr,visitc]=size(visit1);
nb_pts=(size(mf,'c'))*(size(mf,'r'))-visitc;


//when we're in a baggin, we have to go back
path(1)=l+%i*c;
starting_pt=0;


disp('Be a bit patient: we have to find the best path in');
disp(nb_pts);
disp('points !');


while (nb_treated_pts<nb_pts);//are there still unvisited pixels ?

   //if all neighbours have been visited then move back
	while sum(mf(l-1:l+1,c-1:c+1))>seuil_mf then
	  starting_pt=starting_pt+1;
	  l=real(path(starting_pt));
	  c=imag(path(starting_pt));
	end;//while sum

	phase=pw(l,c);
	shift1=jumps(l,c);

	[val_min,position]=min(mf(l-1:l+1,c-1:c+1));

	//position of the new integrator point
	l=l+(position(1)-2);
	c=c+(position(2)-2);

	if (phase-pw(l,c))>threshold then 
	  shift1=shift1+1;
	end;//if phase
	
	if (phase-pw(l,c))<-threshold then 
	  shift1=shift1-1;
	end;//if phase
  
	jumps(l,c)=shift1;
	mf(l,c)=vval;
	nb_treated_pts=nb_treated_pts+1;
	path(nb_treated_pts)=l+%i*c;


      end;//while
      
      
//borders are considered identical to
//their immediate neighbours
jumps(1,:)=jumps(2,:);
jumps($,:)=jumps($-1,:);
jumps(:,1)=jumps(:,2);
jumps(:,$)=jumps(:,$-1);
      

//final calculation
puw=pw+jumps;


endfunction



//
// -------------------------------------------------------------------------
// SIP - Scilab Image Processing toolbox
// Copyright (C) 2002-2004  Ricardo Fabbri
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// -------------------------------------------------------------------------
//
